@@ -1,7 +1,7 @@
 /*
  * print.c	Routines to print stuff.
  *
- * Version:	$Id: 421732f67301900ed5df55d4c704cbdd8a3ae9b1 $
+ * Version:	$Id: 43c457c9cecebf3bc4ee8e74c4cdde209c0ff8fe $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000,2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 421732f67301900ed5df55d4c704cbdd8a3ae9b1 $")
+RCSID("$Id: 43c457c9cecebf3bc4ee8e74c4cdde209c0ff8fe $")
 
 #include	<freeradius-devel/libradius.h>
 
@@ -569,7 +569,7 @@ size_t vp_prints_value_json(char *out, size_t outlen, VALUE_PAIR const *vp)
 					break;
 
 				case '\n':
-					*out++ = 'b';
+					*out++ = 'n';
 					freespace--;
 					break;
 
@@ -583,7 +583,7 @@ size_t vp_prints_value_json(char *out, size_t outlen, VALUE_PAIR const *vp)
 					freespace--;
 					break;
 				default:
-					len = snprintf(out, freespace, "u%04X", *q);
+					len = snprintf(out, freespace, "u%04X", (uint8_t) *q);
 					if (is_truncated(len, freespace)) return (outlen - freespace) + len;
 					out += len;
 					freespace -= len;
