@@ -23,7 +23,7 @@
  * @copyright 2012  The FreeRADIUS server project
  * @copyright 2012  Alan DeKok <aland@deployingradius.com>
  */
-RCSID("$Id: 81c89635d59e93d30dc646fbd84f8ba146fab3f6 $")
+RCSID("$Id: 56e8f87e68282570c32183048d051925f1b538e8 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/heap.h>
@@ -643,10 +643,12 @@ static int fr_connection_pool_check(fr_connection_pool_t *pool)
 	 *	have fewer than "min".  When that happens, open more
 	 *	connections to enforce "min".
 	 */
-	if ((pool->num + pool->pending) <= pool->min) {
+	if ((pool->num + pool->pending) < pool->min) {
 		spawn = pool->min - (pool->num + pool->pending);
 		extra = 0;
 
+		INFO("Need %i more connections to reach min connections (%i)", spawn, pool->min);
+
 	/*
 	 *	If we're about to create more than "max",
 	 *	don't create more.
@@ -666,7 +668,7 @@ static int fr_connection_pool_check(fr_connection_pool_t *pool)
 	 *	AND we don't have enough idle connections.
 	 *	Open some more.
 	 */
-	} else if (idle <= pool->spare) {
+	} else if (idle < pool->spare) {
 		/*
 		 *	Not enough spare connections.  Spawn a few.
 		 *	But cap the pool size at "max"
@@ -678,6 +680,8 @@ static int fr_connection_pool_check(fr_connection_pool_t *pool)
 			spawn = pool->max - (pool->num + pool->pending);
 		}
 
+		INFO("Need %i more connections to reach %i spares", spawn, pool->spare);
+
 	/*
 	 *	min < num < max
 	 *
@@ -712,8 +716,6 @@ static int fr_connection_pool_check(fr_connection_pool_t *pool)
 	 *	a connection. Avoids spurious log messages.
 	 */
 	if (spawn) {
-		INFO("%s: Need %i more connections to reach %i spares",
-		     pool->log_prefix, spawn, pool->spare);
 		pthread_mutex_unlock(&pool->mutex);
 		fr_connection_spawn(pool, now, false); /* ignore return code */
 		pthread_mutex_lock(&pool->mutex);
@@ -821,11 +823,6 @@ static void *fr_connection_get_internal(fr_connection_pool_t *pool, bool spawn)
 	 */
 	if (!spawn) return NULL;
 
-	/*
-	 *	We don't have a connection.  Try to open a new one.
-	 */
-	rad_assert(pool->active == pool->num);
-
 	if (pool->num == pool->max) {
 		bool complain = false;
 
@@ -838,7 +835,7 @@ static void *fr_connection_get_internal(fr_connection_pool_t *pool, bool spawn)
 		}
 
 		pthread_mutex_unlock(&pool->mutex);
-		
+
 		if (!RATE_LIMIT_ENABLED || complain) {
 			ERROR("%s: No connections available and at max connection limit", pool->log_prefix);
 		}
