@@ -1,7 +1,7 @@
 /*
  * @name modcall.c
  *
- * Version:	$Id: 312b1bb230c927e8ab34eae73977886c9e674fb5 $
+ * Version:	$Id: c385db21f4a53ee6e98897346677a6df32dcdf25 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000,2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 312b1bb230c927e8ab34eae73977886c9e674fb5 $")
+RCSID("$Id: c385db21f4a53ee6e98897346677a6df32dcdf25 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modpriv.h>
@@ -1664,7 +1664,7 @@ int modcall_fixup_update(vp_map_t *map, UNUSED void *ctx)
 
 				if (!map_cast_from_hex(map, T_BARE_WORD, vpt->name)) {
 					map->rhs = vpt;
-					cf_log_err(map->ci, "%s", fr_strerror());
+					cf_log_err(map->ci, "Cannot parse RHS hex as the data type of the attribute %s", map->lhs->tmpl_da->name);
 					return -1;
 				}
 				talloc_free(vpt);
@@ -1684,9 +1684,9 @@ int modcall_fixup_update(vp_map_t *map, UNUSED void *ctx)
 				da = dict_attrbytype(map->lhs->tmpl_da->attr, map->lhs->tmpl_da->vendor,
 						     map->rhs->tmpl_data_type);
 				if (!da) {
-					fr_strerror_printf("Cannot find %s variant of attribute \"%s\"",
-							   fr_int2str(dict_attr_types, map->rhs->tmpl_data_type,
-							   "<INVALID>"), map->lhs->tmpl_da->name);
+					cf_log_err(map->ci, "Cannot find %s variant of attribute \"%s\"",
+						   fr_int2str(dict_attr_types, map->rhs->tmpl_data_type,
+							      "<INVALID>"), map->lhs->tmpl_da->name);
 					return -1;
 				}
 				map->lhs->tmpl_da = da;
@@ -2095,7 +2095,12 @@ static modcallable *do_compile_modxlat(modcallable *parent,
 	memcpy(csingle->actions, defaultactions[component][GROUPTYPE_SIMPLE],
 	       sizeof(csingle->actions));
 
-	mx->xlat_name = strdup(fmt);
+	mx->xlat_name = talloc_strdup(mx, fmt);
+	if (!mx->xlat_name) {
+		talloc_free(mx);
+		return NULL;
+	}
+
 	if (fmt[0] != '%') {
 		char *p;
 		mx->exec = true;
@@ -3324,7 +3329,7 @@ static bool pass2_callback(void *ctx, fr_cond_t *c)
 
 					if (!map_cast_from_hex(map, T_BARE_WORD, vpt->name)) {
 						map->rhs = vpt;
-						cf_log_err(map->ci, "%s", fr_strerror());
+						cf_log_err(map->ci, "Cannot parse RHS hex as the data type of the attribute %s", map->lhs->tmpl_da->name);
 						return -1;
 					}
 					talloc_free(vpt);
