@@ -1,7 +1,7 @@
 /*
  * radclient.c	General radius packet debug tool.
  *
- * Version:	$Id: 7594ed4926ed148234488508327b592cbb467ddf $
+ * Version:	$Id: 2f78c306e9fe6211c21050d5eb41ccabc5a22c6d $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 7594ed4926ed148234488508327b592cbb467ddf $")
+RCSID("$Id: 2f78c306e9fe6211c21050d5eb41ccabc5a22c6d $")
 
 #include <freeradius-devel/radclient.h>
 #include <freeradius-devel/radpaths.h>
@@ -977,8 +977,10 @@ static int send_one_packet(rc_request_t *request)
 		return -1;
 	}
 
-	fr_packet_header_print(fr_log_fp, request->packet, false);
-	if (fr_debug_lvl > 0) vp_printlist(fr_log_fp, request->packet->vps);
+	if (fr_log_fp) {
+		fr_packet_header_print(fr_log_fp, request->packet, false);
+		if (fr_debug_lvl > 0) vp_printlist(fr_log_fp, request->packet->vps);
+	}
 
 	return 0;
 }
@@ -1087,8 +1089,10 @@ static int recv_one_packet(int wait_time)
 		goto packet_done;
 	}
 
-	fr_packet_header_print(fr_log_fp, request->reply, true);
-	if (fr_debug_lvl > 0) vp_printlist(fr_log_fp, request->reply->vps);
+	if (fr_log_fp) {
+		fr_packet_header_print(fr_log_fp, request->reply, true);
+		if (fr_debug_lvl > 0) vp_printlist(fr_log_fp, request->reply->vps);
+	}
 
 	/*
 	 *	Increment counters...
@@ -1621,17 +1625,17 @@ int main(int argc, char **argv)
 	dict_free();
 
 	if (do_summary) {
-		DEBUG("Packet summary:\n"
-		      "\tAccepted      : %" PRIu64 "\n"
-		      "\tRejected      : %" PRIu64 "\n"
-		      "\tLost          : %" PRIu64 "\n"
-		      "\tPassed filter : %" PRIu64 "\n"
-		      "\tFailed filter : %" PRIu64,
-		      stats.accepted,
-		      stats.rejected,
-		      stats.lost,
-		      stats.passed,
-		      stats.failed
+		printf("Packet summary:\n"
+		       "\tAccepted      : %" PRIu64 "\n"
+		       "\tRejected      : %" PRIu64 "\n"
+		       "\tLost          : %" PRIu64 "\n"
+		       "\tPassed filter : %" PRIu64 "\n"
+		       "\tFailed filter : %" PRIu64 "\n",
+		       stats.accepted,
+		       stats.rejected,
+		       stats.lost,
+		       stats.passed,
+		       stats.failed
 		);
 	}
 
