@@ -1,7 +1,7 @@
 /*
  * realms.c	Realm handling code
  *
- * Version:     $Id: 738e69abb94dbf6dbd83486819311127ed812e2e $
+ * Version:     $Id: 4f3c6b80e27d2690405e66619194b8543d681a83 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2007  Alan DeKok <aland@deployingradius.com>
  */
 
-RCSID("$Id: 738e69abb94dbf6dbd83486819311127ed812e2e $")
+RCSID("$Id: 4f3c6b80e27d2690405e66619194b8543d681a83 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/realms.h>
@@ -192,10 +192,17 @@ static int home_pool_name_cmp(void const *one, void const *two)
 }
 
 
-static size_t CC_HINT(nonnull) xlat_cs(CONF_SECTION *cs, char const *fmt, char *out, size_t outlen)
+static size_t xlat_cs(CONF_SECTION *cs, char const *fmt, char *out, size_t outlen)
 {
 	char const *value = NULL;
 
+	if (!fmt) {
+		DEBUG("No configuration item requested.  Ignoring.");
+
+		*out = '\0';
+		return 0;
+	}
+
 	/*
 	 *	Instance name
 	 */
@@ -224,8 +231,8 @@ static size_t CC_HINT(nonnull) xlat_cs(CONF_SECTION *cs, char const *fmt, char *
 /*
  *	Xlat for %{home_server:foo}
  */
-static ssize_t CC_HINT(nonnull) xlat_home_server(UNUSED void *instance, REQUEST *request,
-						 char const *fmt, char *out, size_t outlen)
+static ssize_t xlat_home_server(UNUSED void *instance, REQUEST *request,
+				char const *fmt, char *out, size_t outlen)
 {
 	if (!request->home_server) {
 		RWDEBUG("No home_server associated with this request");
@@ -234,6 +241,13 @@ static ssize_t CC_HINT(nonnull) xlat_home_server(UNUSED void *instance, REQUEST
 		return 0;
 	}
 
+	if (!fmt) {
+		RWDEBUG("No configuration item requested.  Ignoring.");
+
+		*out = '\0';
+		return 0;
+	}
+
 	if (strcmp(fmt, "state") == 0) {
 		char const *state;
 
@@ -266,8 +280,8 @@ static ssize_t CC_HINT(nonnull) xlat_home_server(UNUSED void *instance, REQUEST
 /*
  *	Xlat for %{home_server_pool:foo}
  */
-static ssize_t CC_HINT(nonnull) xlat_server_pool(UNUSED void *instance, REQUEST *request,
-						 char const *fmt, char *out, size_t outlen)
+static ssize_t xlat_server_pool(UNUSED void *instance, REQUEST *request,
+				char const *fmt, char *out, size_t outlen)
 {
 	if (!request->home_pool) {
 		RWDEBUG("No home_pool associated with this request");
@@ -276,6 +290,13 @@ static ssize_t CC_HINT(nonnull) xlat_server_pool(UNUSED void *instance, REQUEST
 		return 0;
 	}
 
+	if (!fmt) {
+		RWDEBUG("No configuration item requested.  Ignoring.");
+
+		*out = '\0';
+		return 0;
+	}
+
 	if (strcmp(fmt, "state") == 0) {
 		char const *state;
 
@@ -412,9 +433,9 @@ void realm_home_server_sanitize(home_server_t *home, CONF_SECTION *cs)
 	FR_INTEGER_BOUND_CHECK("ping_interval", home->ping_interval, <=, 120);
 
 	FR_TIMEVAL_BOUND_CHECK("response_window", &home->response_window, >=, 0, 1000);
-	FR_TIMEVAL_BOUND_CHECK("response_window", &home->response_window, <=, 60, 0);
 	FR_TIMEVAL_BOUND_CHECK("response_window", &home->response_window, <=,
 			       main_config.max_request_time, 0);
+	FR_TIMEVAL_BOUND_CHECK("response_window", &home->response_window, <=, 60, 0);
 
 	FR_INTEGER_BOUND_CHECK("response_timeouts", home->max_response_timeouts, >=, 1);
 	FR_INTEGER_BOUND_CHECK("response_timeouts", home->max_response_timeouts, <=, 1000);
@@ -746,7 +767,9 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 
 		switch (proto) {
 		case IPPROTO_UDP:
+#ifdef WITH_TCP
 			home_servers_udp = true;
+#endif
 			break;
 
 		case IPPROTO_TCP:
@@ -1135,7 +1158,7 @@ void realm_pool_free(home_pool_t *pool)
 	}
 
 	this->next = NULL;
-	this->when = now + 60;
+	this->when = now + 300;
 	this->pool = pool;
 	pthread_mutex_unlock(&pool_free_mutex);
 }
@@ -2443,8 +2466,8 @@ home_server_t *home_server_ldb(char const *realmname,
 			hash = 0;
 			break;
 		}
-		fr_hash_update(&request->packet->src_port,
-				 sizeof(request->packet->src_port), hash);
+		hash = fr_hash_update(&request->packet->src_port,
+				      sizeof(request->packet->src_port), hash);
 		start = hash % pool->num_home_servers;
 		break;
 
