@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: 58bc90803bd2a3a126405cefb842d5d072fde95c $
+ * Version:	$Id: ae5a5ae06c1050c9f0223944a4c565a9624c7b08 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 58bc90803bd2a3a126405cefb842d5d072fde95c $")
+RCSID("$Id: ae5a5ae06c1050c9f0223944a4c565a9624c7b08 $")
 USES_APPLE_DEPRECATED_API	/* OpenSSL API has been deprecated by Apple */
 
 #include <freeradius-devel/radiusd.h>
@@ -222,6 +222,7 @@ static const CONF_PARSER thread_config[] = {
 
 static pthread_mutex_t *ssl_mutexes = NULL;
 
+#ifdef HAVE_CRYPTO_SET_ID_CALLBACK
 static unsigned long ssl_id_function(void)
 {
 	unsigned long ret;
@@ -235,7 +236,9 @@ static unsigned long ssl_id_function(void)
 
 	return ret;
 }
+#endif
 
+#ifdef HAVE_CRYPTO_SET_LOCKING_CALLBACK
 static void ssl_locking_function(int mode, int n, UNUSED char const *file, UNUSED int line)
 {
 	if (mode & CRYPTO_LOCK) {
@@ -244,6 +247,7 @@ static void ssl_locking_function(int mode, int n, UNUSED char const *file, UNUSE
 		pthread_mutex_unlock(&(ssl_mutexes[n]));
 	}
 }
+#endif
 
 static int setup_ssl_mutexes(void)
 {
@@ -259,8 +263,12 @@ static int setup_ssl_mutexes(void)
 		pthread_mutex_init(&(ssl_mutexes[i]), NULL);
 	}
 
+#ifdef HAVE_CRYPTO_SET_ID_CALLBACK
 	CRYPTO_set_id_callback(ssl_id_function);
+#endif
+#ifdef HAVE_CRYPTO_SET_LOCKING_CALLBACK
 	CRYPTO_set_locking_callback(ssl_locking_function);
+#endif
 
 	return 1;
 }
@@ -718,7 +726,11 @@ static void *request_handler_thread(void *arg)
 	 *	must remove the thread's error queue before
 	 *	exiting to prevent memory leaks.
 	 */
+#if OPENSSL_VERSION_NUMBER < 0x10000000L
 	ERR_remove_state(0);
+#elif OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+	ERR_remove_thread_state(NULL);
+#endif
 #endif
 
 	pthread_mutex_lock(&thread_pool.queue_mutex);
@@ -1101,8 +1113,12 @@ void thread_pool_stop(void)
 	 *	We're no longer threaded.  Remove the mutexes and free
 	 *	the memory.
 	 */
+#ifdef HAVE_CRYPTO_SET_ID_CALLBACK
 	CRYPTO_set_id_callback(NULL);
+#endif
+#ifdef HAVE_CRYPTO_SET_LOCKING_CALLBACK
 	CRYPTO_set_locking_callback(NULL);
+#endif
 
 	free(ssl_mutexes);
 #endif
