@@ -1,7 +1,7 @@
 /*
  * tls.c
  *
- * Version:     $Id: 0045ede7ac8cb6b6c6bd8a62365e32ff637641ab $
+ * Version:     $Id: 4f52df4700951cf614fa641ed6ad01ae526d51b3 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 0045ede7ac8cb6b6c6bd8a62365e32ff637641ab $")
+RCSID("$Id: 4f52df4700951cf614fa641ed6ad01ae526d51b3 $")
 USES_APPLE_DEPRECATED_API	/* OpenSSL API has been deprecated by Apple */
 
 #include <freeradius-devel/radiusd.h>
@@ -157,7 +157,6 @@ static int tls_socket_recv(rad_listen_t *listener)
 		rad_assert(sock->packet != NULL);
 		request->packet = talloc_steal(request, sock->packet);
 
-		request->component = "<core>";
 		request->component = "<tls-connect>";
 
 		request->reply = rad_alloc(request, false);
@@ -175,7 +174,7 @@ static int tls_socket_recv(rad_listen_t *listener)
 
 		SSL_set_ex_data(sock->ssn->ssl, FR_TLS_EX_INDEX_REQUEST, (void *)request);
 		SSL_set_ex_data(sock->ssn->ssl, fr_tls_ex_index_certs, (void *) &sock->certs);
-		SSL_set_ex_data(sock->ssn->ssl, FR_TLS_EX_INDEX_TALLOC, sock->parent);
+		SSL_set_ex_data(sock->ssn->ssl, FR_TLS_EX_INDEX_TALLOC, NULL);
 
 		doing_init = true;
 	}
@@ -480,6 +479,34 @@ int dual_tls_send(rad_listen_t *listener, REQUEST *request)
 	return 0;
 }
 
+static int try_connect(tls_session_t *ssn)
+{
+	int ret;
+	ret = SSL_connect(ssn->ssl);
+	if (ret < 0) {
+		switch (SSL_get_error(ssn->ssl, ret)) {
+			default:
+				break;
+
+
+
+		case SSL_ERROR_WANT_READ:
+		case SSL_ERROR_WANT_WRITE:
+			ssn->connected = false;
+			return 0;
+		}
+	}
+
+	if (ret <= 0) {
+		tls_error_io_log(NULL, ssn, ret, "Failed in " STRINGIFY(__FUNCTION__) " (SSL_connect)");
+		talloc_free(ssn);
+
+		return -1;
+	}
+
+	return 1;
+}
+
 
 #ifdef WITH_PROXY
 /*
@@ -501,6 +528,18 @@ static ssize_t proxy_tls_read(rad_listen_t *listener)
 	uint8_t *data;
 	listen_socket_t *sock = listener->data;
 
+	if (!sock->ssn->connected) {
+		rcode = try_connect(sock->ssn);
+		if (rcode == 0) return 0;
+
+		if (rcode < 0) {
+			SSL_shutdown(sock->ssn->ssl);
+			return -1;
+		}
+
+		sock->ssn->connected = true;
+	}
+
 	/*
 	 *	Get the maximum size of data to receive.
 	 */
@@ -694,6 +733,20 @@ int proxy_tls_send(rad_listen_t *listener, REQUEST *request)
 						request);
 	}
 
+	if (!sock->ssn->connected) {
+		PTHREAD_MUTEX_LOCK(&sock->mutex);
+		rcode = try_connect(sock->ssn);
+		PTHREAD_MUTEX_UNLOCK(&sock->mutex);
+		if (rcode == 0) return 0;
+
+		if (rcode < 0) {
+			SSL_shutdown(sock->ssn->ssl);
+			return -1;
+		}
+
+		sock->ssn->connected = true;
+	}
+
 	DEBUG3("Proxy is writing %u bytes to SSL",
 	       (unsigned int) request->proxy->data_len);
 	PTHREAD_MUTEX_LOCK(&sock->mutex);
