@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: d52701550a321c91434200b185b49935f58fa468 $
+ * $Id: 59fb6b9ff6dd68fc3fade1eb95d2eb31c0396dcb $
  *
  * @file xlat.c
  * @brief String expansion ("translation"). Implements %Attribute -> value
@@ -24,7 +24,7 @@
  * @copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: d52701550a321c91434200b185b49935f58fa468 $")
+RCSID("$Id: 59fb6b9ff6dd68fc3fade1eb95d2eb31c0396dcb $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/parser.h>
@@ -1623,10 +1623,10 @@ static void xlat_tokenize_debug(xlat_exp_t const *node, int lvl)
 #endif
 
 		case XLAT_ALTERNATE:
-			DEBUG("%.*sif {", lvl, xlat_tabs);
+			DEBUG("%.*sXLAT-IF {", lvl, xlat_tabs);
 			xlat_tokenize_debug(node->child, lvl + 1);
 			DEBUG("%.*s}", lvl, xlat_tabs);
-			DEBUG("%.*selse {", lvl, xlat_tabs);
+			DEBUG("%.*sXLAT-ELSE {", lvl, xlat_tabs);
 			xlat_tokenize_debug(node->alternate, lvl + 1);
 			DEBUG("%.*s}", lvl, xlat_tabs);
 			break;
@@ -1787,7 +1787,10 @@ static ssize_t xlat_tokenize_request(REQUEST *request, char const *fmt, xlat_exp
 	 *	much faster.
 	 */
 	tokens = talloc_typed_strdup(request, fmt);
-	if (!tokens) return -1;
+	if (!tokens) {
+		error = "Out of memory";
+		return -1;
+	}
 
 	slen = xlat_tokenize_literal(request, tokens, head, false, &error);
 
@@ -1806,7 +1809,8 @@ static ssize_t xlat_tokenize_request(REQUEST *request, char const *fmt, xlat_exp
 	 */
 	if (slen < 0) {
 		talloc_free(tokens);
-		rad_assert(error != NULL);
+
+		if (!error) error = "Unknown error";
 
 		REMARKER(fmt, -slen, error);
 		return slen;
@@ -2117,7 +2121,7 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 		 *	Don't escape this.
 		 */
 	case XLAT_LITERAL:
-		XLAT_DEBUG("xlat_aprint LITERAL");
+		XLAT_DEBUG("%.*sxlat_aprint LITERAL", lvl, xlat_spaces);
 		return talloc_typed_strdup(ctx, node->fmt);
 
 		/*
@@ -2129,15 +2133,18 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 		size_t freespace = 256;
 		struct tm ts;
 		time_t when;
+		int usec;
 
-		XLAT_DEBUG("xlat_aprint PERCENT");
+		XLAT_DEBUG("%.*sxlat_aprint PERCENT", lvl, xlat_spaces);
 
 		str = talloc_array(ctx, char, freespace); /* @todo do better allocation */
 		p = node->fmt;
 
 		when = request->timestamp;
+		usec = 0;
 		if (request->packet) {
 			when = request->packet->timestamp.tv_sec;
+			usec = request->packet->timestamp.tv_usec;
 		}
 
 		switch (*p) {
@@ -2199,7 +2206,9 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 
 		case 'T': /* request timestamp */
 			if (!localtime_r(&when, &ts)) goto error;
-			strftime(str, freespace, "%Y-%m-%d-%H.%M.%S.000000", &ts);
+			nl = str + strftime(str, freespace, "%Y-%m-%d-%H.%M.%S", &ts);
+			rad_assert(((str + freespace) - nl) >= 8);
+			snprintf(nl, (str + freespace) - nl, ".%06d",  usec);
 			break;
 
 		case 'Y': /* request year */
@@ -2225,15 +2234,15 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 		break;
 
 	case XLAT_ATTRIBUTE:
-		XLAT_DEBUG("xlat_aprint ATTRIBUTE");
+		XLAT_DEBUG("%.*sxlat_aprint ATTRIBUTE", lvl, xlat_spaces);
 
 		/*
 		 *	Some attributes are virtual <sigh>
 		 */
 		str = xlat_getvp(ctx, request, &node->attr, escape ? false : true, true);
 		if (str) {
-			XLAT_DEBUG("EXPAND attr %s", node->attr.tmpl_da->name);
-			XLAT_DEBUG("       ---> %s", str);
+			XLAT_DEBUG("%.*sEXPAND attr %s", lvl, xlat_spaces, node->attr.tmpl_da->name);
+			XLAT_DEBUG("%.*s       ---> %s", lvl ,xlat_spaces, str);
 		}
 		break;
 
@@ -2327,27 +2336,42 @@ static char *xlat_aprint(TALLOC_CTX *ctx, REQUEST *request, xlat_exp_t const * c
 
 #ifdef HAVE_REGEX
 	case XLAT_REGEX:
-		XLAT_DEBUG("xlat_aprint REGEX");
+		XLAT_DEBUG("%.*sxlat_aprint REGEX", lvl, xlat_spaces);
 		if (regex_request_to_sub(ctx, &str, request, node->attr.tmpl_num) < 0) return NULL;
 
 		break;
 #endif
 
 	case XLAT_ALTERNATE:
-		XLAT_DEBUG("xlat_aprint ALTERNATE");
+		XLAT_DEBUG("%.*sxlat_aprint ALTERNATE", lvl, xlat_spaces);
 		rad_assert(node->child != NULL);
 		rad_assert(node->alternate != NULL);
 
-		str = xlat_aprint(ctx, request, node->child, escape, escape_ctx, lvl);
-		if (str) {
-			XLAT_DEBUG("ALTERNATE got string: %s", str);
-			break;
+		/*
+		 *	If there are no "next" nodes, call ourselves
+		 *	recursively, which is fast.
+		 *
+		 *	If there are "next" nodes, call xlat_process()
+		 *	which does a ton more work.
+		 */
+		if (!node->next) {
+			str = xlat_aprint(ctx, request, node->child, escape, escape_ctx, lvl);
+			if (str) {
+				XLAT_DEBUG("%.*sALTERNATE got first string: %s", lvl, xlat_spaces, str);
+			} else {
+				str = xlat_aprint(ctx, request, node->alternate, escape, escape_ctx, lvl);
+				XLAT_DEBUG("%.*sALTERNATE got alternate string %s", lvl, xlat_spaces, str);
+			}
+		} else {
+
+			if (xlat_process(&str, request, node->child, escape, escape_ctx) > 0) {
+				XLAT_DEBUG("%.*sALTERNATE got first string: %s", lvl, xlat_spaces, str);
+			} else {
+				(void) xlat_process(&str, request, node->alternate, escape, escape_ctx);
+				XLAT_DEBUG("%.*sALTERNATE got alternate string %s", lvl, xlat_spaces, str);
+			}
 		}
-
-		XLAT_DEBUG("ALTERNATE going to alternate");
-		str = xlat_aprint(ctx, request, node->alternate, escape, escape_ctx, lvl);
 		break;
-
 	}
 
 	/*
@@ -2613,10 +2637,12 @@ ssize_t radius_xlat_struct(char *out, size_t outlen, REQUEST *request, xlat_exp_
 
 ssize_t radius_axlat(char **out, REQUEST *request, char const *fmt, xlat_escape_t escape, void *ctx)
 {
+	*out = NULL;
 	return xlat_expand(out, 0, request, fmt, escape, ctx);
 }
 
 ssize_t radius_axlat_struct(char **out, REQUEST *request, xlat_exp_t const *xlat, xlat_escape_t escape, void *ctx)
 {
+	*out = NULL;
 	return xlat_expand_struct(out, 0, request, xlat, escape, ctx);
 }
