@@ -15,13 +15,13 @@
  */
 
 /**
- * $Id: cc9bd6c5cc74fd26a0c78a3eaf3bdc719e58b917 $
+ * $Id: 801e6ada2b7143fee4c647eb732a686d01c72c36 $
  * @file rlm_detail.c
  * @brief Write plaintext versions of packets to flatfiles.
  *
  * @copyright 2000,2006  The FreeRADIUS server project
  */
-RCSID("$Id: cc9bd6c5cc74fd26a0c78a3eaf3bdc719e58b917 $")
+RCSID("$Id: 801e6ada2b7143fee4c647eb732a686d01c72c36 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -224,6 +224,11 @@ static int detail_write(FILE *out, rlm_detail_t *inst, REQUEST *request, RADIUS_
 	VALUE_PAIR *vp;
 	char timestamp[256];
 
+	if (!packet->vps) {
+		RWDEBUG("Skipping empty packet");
+		return 0;
+	}
+
 	if (radius_xlat(timestamp, sizeof(timestamp), request, inst->header, NULL, NULL) < 0) {
 		return -1;
 	}
@@ -421,7 +426,7 @@ skip_group:
 	 *	Flush everything
 	 */
 	fclose(outfp);
-	exfile_unlock(inst->ef, outfd); /* do NOT close outfp */
+	exfile_unlock(inst->ef, outfd); /* do NOT close outfd */
 
 	/*
 	 *	And everything is fine.
