@@ -1,7 +1,7 @@
 /*
  * mem.c  Memory allocation, deallocation stuff.
  *
- * Version:     $Id: c0899f0c8799ea98d1c5d71eeb9ce84afc34468d $
+ * Version:     $Id: 4246dfb22c57378dc8dcd5b6782695892f1b394f $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2001  hereUare Communications, Inc. <raghud@hereuare.com>
  */
 
-RCSID("$Id: c0899f0c8799ea98d1c5d71eeb9ce84afc34468d $")
+RCSID("$Id: 4246dfb22c57378dc8dcd5b6782695892f1b394f $")
 
 #include <stdio.h>
 #include "rlm_eap.h"
@@ -90,8 +90,6 @@ static int _eap_handler_free(eap_handler_t *handler)
 	handler->opaque = NULL;
 	handler->free_opaque = NULL;
 
-	if (handler->certs) fr_pair_list_free(&handler->certs);
-
 	/*
 	 *	Give helpful debug messages if:
 	 *
@@ -111,8 +109,6 @@ static int _eap_handler_free(eap_handler_t *handler)
 		WARN("!! Please read http://wiki.freeradius.org/guide/Certificate_Compatibility     !!");
 		WARN("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
 	}
-
-	talloc_free(handler);
 	
 	return 0;
 }
@@ -436,7 +432,7 @@ eap_handler_t *eaplist_find(rlm_eap_t *inst, REQUEST *request,
 	 *	Might not have been there.
 	 */
 	if (!handler) {
-		ERROR("rlm_eap (%s): No EAP session matching state "
+		RERROR("rlm_eap (%s): No EAP session matching state "
 		       "0x%02x%02x%02x%02x%02x%02x%02x%02x",
 		       inst->xlat_name,
 		       state->vp_octets[0], state->vp_octets[1],
@@ -447,7 +443,7 @@ eap_handler_t *eaplist_find(rlm_eap_t *inst, REQUEST *request,
 	}
 
 	if (handler->trips >= 50) {
-		ERROR("rlm_eap (%s): Aborting! More than 50 roundtrips "
+		RERROR("rlm_eap (%s): Aborting! More than 50 roundtrips "
 		       "made in session with state "
 		       "0x%02x%02x%02x%02x%02x%02x%02x%02x",
 		       inst->xlat_name,
