@@ -1,7 +1,7 @@
 /*
  * eap_fast.c  contains the interfaces that are called from the main handler
  *
- * Version:     $Id: 32380c4580b702094f55e193baa1f516e10c8cd7 $
+ * Version:     $Id: 95e52171885cd7195a2af2fdbeed8abd034020d0 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *   Copyright 2016 The FreeRADIUS server project
  */
 
-RCSID("$Id: 32380c4580b702094f55e193baa1f516e10c8cd7 $")
+RCSID("$Id: 95e52171885cd7195a2af2fdbeed8abd034020d0 $")
 
 #include "eap_fast.h"
 #include "eap_fast_crypto.h"
@@ -951,10 +951,9 @@ static PW_CODE eap_fast_eap_payload(REQUEST *request, eap_handler_t *eap_session
 	if (t->stage == AUTHENTICATION) {	/* FIXME do this only for MSCHAPv2 */
 		VALUE_PAIR *tvp;
 
-        RWDEBUG2("AUTHENTICATION");
-        vp = fr_pair_make(fake, &fake->config, "EAP-Type", "0", T_OP_EQ);
-        vp->vp_integer = t->default_method;
-        RWDEBUG2("AUTHENTICATION");
+		RDEBUG2("AUTHENTICATION");
+		vp = fr_pair_make(fake, &fake->config, "EAP-Type", "0", T_OP_EQ);
+		vp->vp_integer = t->default_method;
 
 		/*
 		 * RFC 5422 section 3.2.3 - Authenticating Using EAP-FAST-MSCHAPv2
@@ -1236,9 +1235,6 @@ PW_CODE eap_fast_process(eap_handler_t *eap_session, tls_session_t *tls_session)
 
 		eap_fast_append_result(tls_session, code);
 
-		if (code == PW_CODE_ACCESS_REJECT)
-			break;
-
 		if (t->pac.send) {
 			RDEBUG("Peer requires new PAC");
 			eap_fast_send_pac_tunnel(request, tls_session);
