@@ -1,7 +1,7 @@
 /*
  * rlm_eap_peap.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: e43c9c72cc0983cbc60f6b7f10522ad87812c082 $
+ * Version:     $Id: 5bf7c9631df978040a3afd68a572013363dfaef0 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2006 The FreeRADIUS server project
  */
 
-RCSID("$Id: e43c9c72cc0983cbc60f6b7f10522ad87812c082 $")
+RCSID("$Id: 5bf7c9631df978040a3afd68a572013363dfaef0 $")
 
 #include "eap_peap.h"
 
@@ -262,6 +262,13 @@ static int mod_process(void *arg, eap_handler_t *handler)
 		peap = tls_session->opaque = peap_alloc(tls_session, inst);
 	}
 
+	/*
+	 *	Negotiate PEAP versions down.
+	 */
+	if ((handler->eap_ds->response->type.data[0] & 0x03) < tls_session->peap_flag) {
+		tls_session->peap_flag = handler->eap_ds->response->type.data[0] & 0x03;
+	}
+
 	status = eaptls_process(handler);
 	if ((status == FR_TLS_INVALID) || (status == FR_TLS_FAIL)) {
 		REDEBUG("[eaptls process] = %s", fr_int2str(fr_tls_status_table, status, "<INVALID>"));
