@@ -15,14 +15,14 @@
  */
 
 /**
- * $Id: 0b76d9a4e666d405e0dff2be449816e034671bab $
+ * $Id: c94831da322fefbcfaa20bbe9b0ea345ab616026 $
  * @file rlm_sql_sqlite.c
  * @brief SQLite driver.
  *
  * @copyright 2013 Network RADIUS SARL <info@networkradius.com>
  * @copyright 2007 Apple Inc.
  */
-RCSID("$Id: 0b76d9a4e666d405e0dff2be449816e034671bab $")
+RCSID("$Id: c94831da322fefbcfaa20bbe9b0ea345ab616026 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -678,9 +678,7 @@ static sql_rcode_t sql_fetch_row(rlm_sql_handle_t *handle, rlm_sql_config_t *con
 	/*
 	 *	No more rows to process (were done)
 	 */
-	if (status == SQLITE_DONE) {
-		return 1;
-	}
+	if (status == SQLITE_DONE) return RLM_SQL_NO_MORE_ROWS;
 
 	/*
 	 *	We only need to do this once per result set, because
@@ -732,7 +730,7 @@ static sql_rcode_t sql_fetch_row(rlm_sql_handle_t *handle, rlm_sql_config_t *con
 		}
 	}
 
-	return 0;
+	return RLM_SQL_OK;
 }
 
 static sql_rcode_t sql_free_result(rlm_sql_handle_t *handle, UNUSED rlm_sql_config_t *config)
@@ -754,7 +752,7 @@ static sql_rcode_t sql_free_result(rlm_sql_handle_t *handle, UNUSED rlm_sql_conf
 	 *	It's just the last error that occurred processing the
 	 *	statement.
 	 */
-	return 0;
+	return RLM_SQL_OK;
 }
 
 /** Retrieves any errors associated with the connection handle
@@ -795,9 +793,7 @@ static int sql_affected_rows(rlm_sql_handle_t *handle,
 {
 	rlm_sql_sqlite_conn_t *conn = handle->conn;
 
-	if (conn->db) {
-		return sqlite3_changes(conn->db);
-	}
+	if (conn->db) return sqlite3_changes(conn->db);
 
 	return -1;
 }
