@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: 643d68ab4de64c73aa2821039e9f15b759ac8f67 $
+ * Version:	$Id: 05d0bc245643e26a2f39aa30e24c5b6609d8c10c $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  * Copyright 2001  Chad Miller <cmiller@surfsouth.com>
  */
 
-RCSID("$Id: 643d68ab4de64c73aa2821039e9f15b759ac8f67 $")
+RCSID("$Id: 05d0bc245643e26a2f39aa30e24c5b6609d8c10c $")
 
 #include	<freeradius-devel/radiusd.h>
 #include	<freeradius-devel/rad_assert.h>
@@ -49,53 +49,11 @@ const FR_NAME_NUMBER sql_rcode_table[] = {
 	{ "server error",	RLM_SQL_ERROR		},
 	{ "query invalid",	RLM_SQL_QUERY_INVALID	},
 	{ "no connection",	RLM_SQL_RECONNECT	},
+	{ "no more rows",	RLM_SQL_NO_MORE_ROWS	},
 	{ NULL, 0 }
 };
 
 
-void *mod_conn_create(TALLOC_CTX *ctx, void *instance)
-{
-	int rcode;
-	rlm_sql_t *inst = instance;
-	rlm_sql_handle_t *handle;
-
-	/*
-	 *	Connections cannot be alloced from the inst or
-	 *	pool contexts due to threading issues.
-	 */
-	handle = talloc_zero(ctx, rlm_sql_handle_t);
-	if (!handle) return NULL;
-
-	handle->log_ctx = talloc_pool(handle, 2048);
-	if (!handle->log_ctx) {
-		talloc_free(handle);
-		return NULL;
-	}
-
-	/*
-	 *	Handle requires a pointer to the SQL inst so the
-	 *	destructor has access to the module configuration.
-	 */
-	handle->inst = inst;
-
-	rcode = (inst->module->sql_socket_init)(handle, inst->config);
-	if (rcode != 0) {
-	fail:
-		/*
-		 *	Destroy any half opened connections.
-		 */
-		talloc_free(handle);
-		return NULL;
-	}
-
-	if (inst->config->connect_query) {
-		if (rlm_sql_select_query(inst, NULL, &handle, inst->config->connect_query) != RLM_SQL_OK) goto fail;
-		(inst->module->sql_finish_select_query)(handle, inst->config);
-	}
-
-	return handle;
-}
-
 /*************************************************************************
  *
  *	Function: sql_fr_pair_list_afrom_str
@@ -489,7 +447,7 @@ int sql_getvpdata(TALLOC_CTX *ctx, rlm_sql_t *inst, REQUEST *request, rlm_sql_ha
 	rcode = rlm_sql_select_query(inst, request, handle, query);
 	if (rcode != RLM_SQL_OK) return -1; /* error handled by rlm_sql_select_query */
 
-	while (rlm_sql_fetch_row(inst, request, handle) == 0) {
+	while (rlm_sql_fetch_row(inst, request, handle) == RLM_SQL_OK) {
 		row = (*handle)->row;
 		if (!row) break;
 		if (sql_fr_pair_list_afrom_str(ctx, request, pair, row) != 0) {
